'use strict';

var core = require('@vueuse/core');
var vueDemi = require('vue-demi');

/**
 * Stopwatch persisted in storage.
 * @param key Key to reference the stopwatch in storage.
 * @param options
 */
function useStopwatch(key, options = {}) {
    const { immediate = false, interval = 'requestAnimationFrame' } = options;
    const { timestamp: now } = core.useTimestamp({ interval, controls: true });
    const startTimestamp = core.useStorage(`stopwatch-timestamp-${key}`, now.value);
    const pauseTimestamp = core.useStorage(`stopwatch-pause-${key}`, immediate ? 0 : now.value);
    const offset = core.useStorage(`stopwatch-offset${key}`, 0);
    const running = vueDemi.computed(() => !pauseTimestamp.value);
    const currentPause = vueDemi.computed(() => !running.value ? now.value - pauseTimestamp.value : 0);
    const elapsed = vueDemi.computed(() => now.value - startTimestamp.value - offset.value - currentPause.value);
    /**
     * Pauses the stopwatch.
     */
    function pause() {
        if (pauseTimestamp.value)
            return;
        pauseTimestamp.value = now.value;
    }
    /**
     * Resumes the stopwatch.
     */
    function resume() {
        offset.value = offset.value + currentPause.value;
        pauseTimestamp.value = 0;
    }
    /**
     * Resets the stopwatch to 0.
     */
    function reset() {
        startTimestamp.value = now.value;
        offset.value = 0;
        if (pauseTimestamp.value)
            pauseTimestamp.value = now.value;
    }
    return {
        pause,
        resume,
        reset,
        elapsed,
        running
    };
}

module.exports = useStopwatch;
