(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('@vueuse/core'), require('vue-demi')) :
  typeof define === 'function' && define.amd ? define(['@vueuse/core', 'vue-demi'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global["use-persistent-stopwatch"] = factory(global.VueUse, global.VueDemi));
})(this, (function (core, vueDemi) { 'use strict';

  /**
   * Stopwatch persisted in storage.
   * @param key Key to reference the stopwatch in storage.
   * @param options
   */
  function useStopwatch(key, options = {}) {
      const { immediate = false, interval = 'requestAnimationFrame' } = options;
      const { timestamp: now } = core.useTimestamp({ interval, controls: true });
      const startTimestamp = core.useStorage(`stopwatch-timestamp-${key}`, now.value);
      const pauseTimestamp = core.useStorage(`stopwatch-pause-${key}`, immediate ? 0 : now.value);
      const offset = core.useStorage(`stopwatch-offset${key}`, 0);
      const running = vueDemi.computed(() => !pauseTimestamp.value);
      const currentPause = vueDemi.computed(() => !running.value ? now.value - pauseTimestamp.value : 0);
      const elapsed = vueDemi.computed(() => now.value - startTimestamp.value - offset.value - currentPause.value);
      /**
       * Pauses the stopwatch.
       */
      function pause() {
          if (pauseTimestamp.value)
              return;
          pauseTimestamp.value = now.value;
      }
      /**
       * Resumes the stopwatch.
       */
      function resume() {
          offset.value = offset.value + currentPause.value;
          pauseTimestamp.value = 0;
      }
      /**
       * Resets the stopwatch to 0.
       */
      function reset() {
          startTimestamp.value = now.value;
          offset.value = 0;
          if (pauseTimestamp.value)
              pauseTimestamp.value = now.value;
      }
      return {
          pause,
          resume,
          reset,
          elapsed,
          running
      };
  }

  return useStopwatch;

}));
